/**
 * 
 */
package reseptit;


import java.io.OutputStream;
import java.io.PrintStream;

/**
 * <pre>
 *|------------------------------------------------------------------------|
 *| Luokan nimi:  Ainesosa                             | Avustajat:        |
 *|-------------------------------------------------------------------------
 *| Vastuualueet:                                      |                   |
 *|                                                    |                   |
 *| - Tietää ainesosan kentät (nimi, määrä jne)        |                   |
 *| - osaa muuttaa 1| jauheliha | ..| merkkijonoksi    |                   |
 *|-------------------------------------------------------------------------
 * </pre>
 * @author Omistaja
 * @version 17.3.2025
 *
 */
public class Ainesosa {

        private String nimi = "";
        private int id = 0;
        private int ohjeNro;
        private double maara;
        private String yksikko;
        
        private static int seuraavaId = 1;
    
        
        /**
         * Parametriton muodostaja
         */
        public Ainesosa() {
            valitseId();
        }
        
        
        /**
         * Muodostaja, jolle annetaan nimi
         * @param nimi ainesosalle annettava nimi
         */
        public Ainesosa(String nimi) {
            this.nimi = nimi;
            valitseId();
        }
        
        
        /**
         * Yhdistetään ainesosa ohjeeseen  
         * @param ohjeNro ohjeen viitenumero 
         */
        public Ainesosa(int ohjeNro) {
            this.ohjeNro = ohjeNro;
        }
        
        
        /**
         * Apumetodi, jolla saadaan täytettyä testiarvot Ainesosalle.
         * @param nro viite ohjeeseen, johon ainesosaa tarvitaan
         */
        public void vastaaMaito(int nro) {
            ohjeNro = nro;
            nimi = "Maito";
            yksikko = "dl";
            maara = 2.5;
        }
        
        
        /**
         * Apumetodi, jolla saadaan täytettyä testiarvot Ainesosalle.
         * @param nro viite ohjeeseen, johon ainesosaa tarvitaan
         */
        public void vastaaSuola(int nro) {
            ohjeNro = nro;
            nimi = "Suola";
            yksikko = "tl";
            maara = 0.5;
        }
        
        
        /**
         * Apumetodi, jolla saadaan täytettyä testiarvot Ainesosalle.
         * @param nro viite ohjeeseen, johon ainesosaa tarvitaan
         */
        public void vastaaVehnajauho(int nro) {
            ohjeNro = nro;
            nimi = "Vehnajauho";
            yksikko = "dl";
            maara = 8;
        }


        
        /**
         * @param ain kysyttävä ainesosa
         * @return palauttaa ainesosan nimen
         */
        public String getNimi(Ainesosa ain) {
            return this.nimi;
        }
        
        
        /**
         * @param ain ainesosa
         * @return palauttaa määrän
         */
        public double getMaara(Ainesosa ain) {
            return this.maara;
        }
        
        
        /**
         * @param ain ainesosa
         * @return palauttaa ainesosan yksikön
         */
        public String getYksikko(Ainesosa ain) {
            return this.yksikko;
        }
                
        /**
         * Tulostaa ainesosan tiedot
         * @param os tietovirta mihin tulostetaan
         */
        public void tulosta(OutputStream os) {
            tulosta(new PrintStream(os));
        }
        
        
        /**
         * Tulostaa ainesosan tiedot
         * @param out tietovirta mihin tulostetaan
         */
        public void tulosta(PrintStream out) {
            out.println(String.format("%03d", id) + " " + nimi + " " + maara + " " + yksikko);
        }
        
        
        /**
         * Antaa ainesosalle tunnusnumeron
         * @return palauttaa ainesosan id:n
         * @example
         * <pre name="test">
         * Ainesosa maito3 = new Ainesosa();
         * Ainesosa maito4 = new Ainesosa();
         * maito3.getId() === 0;
         * maito3.lisaa();
         * maito4.lisaa();
         * int n1 = maito3.getId();
         * int n2 = maito4.getId();
         * n1 === n2-1;
         * </pre>
         */
        public int valitseId() {
            this.id = seuraavaId;
            seuraavaId++;
            return this.id;
        }
        
        
        /**
         * Poimii aineosan numeron
         * @return palauttaa ainesosan numeron
         */
        public int getId() {
            return id;
        }
        
        
        /**
         * Palautetaan missä ohjeessa ainesosaa käytetään
         * @return ohjeen id
         */
        public int getOhjeNro() {
            return ohjeNro;
        }

    
    /**
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Ainesosa ain = new Ainesosa();
        ain.vastaaMaito(2);
        ain.tulosta(System.out);
    
    }

}
