/**
 * 
 */
package reseptit;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

/**
 * <pre>
 *|------------------------------------------------------------------------|
 *| Luokan nimi:  Ainesosat                            | Avustajat:        |
 *|-------------------------------------------------------------------------
 *| Vastuualueet:                                      |                   |
 *|                                                    | - Ainesosa        |
 *| - Pitää listaa ainesosista, sekä osaa lisätä ja    |                   |
 *|  poistaa ainesosia                                 |                   |
 *| - Osaa etisä ainesosia                             |                   |
 *| - Lukee ja kirjoittaa ohjeista tiedostoon          |                   |
 *|-------------------------------------------------------------------------
 * </pre>
 * @author Omistaja
 * @version 17.3.2025
 *
 */
public class Ainesosat implements Iterable<Ainesosa>{
    
    private String tiedNimi = "";
    
    private final Collection<Ainesosa> alkiot  = new ArrayList<Ainesosa>();
    
    
    /**
     * Joku konstruktori
     */
    public Ainesosat() {
        
    }
    
    
    /**
     * Lisää uuden ainesosan tietorakenteeseen.  Ottaa ainesosan omistukseensa.
     * @param ain lisättävä ainesosa.  Huom tietorakenne muuttuu omistajaksi
     */
    public void lisaa(Ainesosa ain) {
        alkiot.add(ain);
    }
    
    
    /**
     * Lukee Ainesosat tiedostosta.
     * @param hakemisto tiedoston hakemisto
     * @throws SailoException jos lukeminen epäonnistuu
     */
    public void lueTiedostosta(String hakemisto) throws SailoException {
        tiedNimi = hakemisto + "/ainesosat.dat";
        throw new SailoException("Ei osata vielä lukea tiedostoa " + tiedNimi);
    }


    /**
     * Tallentaa ainesosat tiedostoon.
     * @throws SailoException jos talletus epäonnistuu
     */
    public void talleta() throws SailoException {
        throw new SailoException("Ei osata vielä tallettaa tiedostoa " + tiedNimi);
    }


    /**
     * Palauttaa Ainesosien lukumäärän
     * @return ainesosien lukumäärä
     */
    public int getALkm() {
        return alkiot.size();
    }

    
    /**
     * Iteraattori kaikkien ainesosien läpikäymiseen
     * @return ainesosaiteraattori
     * 
     * @example
     * <pre name="test">
     * #PACKAGEIMPORT
     * #import java.util.*;
     * 
     *  Ainesosat aineet = new Ainesosat();
     *  Ainesosa maito21 = new Ainesosa(2); aineet.lisaa(maito21);
     *  Ainesosa maito11 = new Ainesosa(1); aineet.lisaa(maito11);
     *  Ainesosa maito22 = new Ainesosa(2); aineet.lisaa(maito22);
     *  Ainesosa maito12 = new Ainesosa(1); aineet.lisaa(maito12);
     *  Ainesosa maito23 = new Ainesosa(2); aineet.lisaa(maito23);
     * 
     *  Iterator<Ainesosa> i2=aineet.iterator();
     *  i2.next() === maito21;
     *  i2.next() === maito11;
     *  i2.next() === maito22;
     *  i2.next() === maito12;
     *  i2.next() === maito23;
     *  i2.next() === maito12;  #THROWS NoSuchElementException  
     *  
     *  int n = 0;
     *  int anrot[] = {2,1,2,1,2};
     *  
     *  for ( Ainesosa ain:aineet ) { 
     *    aine.getOhjeNro() === anrot[n]; n++;  
     *  }
     *  
     *  n === 5;
     *  
     * </pre>
     */
    @Override
    public Iterator<Ainesosa> iterator() {
        return alkiot.iterator();
    }
    
    
    /**
     * Haetaan kaikki ohjeen ainesosat
     * @param tunnusnro ohjeen tunnusnumero jolle ainesosia haetaan
     * @return tietorakenne jossa viiteet löydetteyihin ainesosiin
     * @example
     * <pre name="test">
     * #import java.util.*;
     * 
     *  Ainesosat aineet = new Ainesosat();
     *  Ainesosa maito21 = new Ainesosa(2); aineet.lisaa(maito21);
     *  Ainesosa maito11 = new Ainesosa(1); aineet.lisaa(maito11);
     *  Ainesosa maito22 = new Ainesosa(2); aineet.lisaa(maito22);
     *  Ainesosa maito12 = new Ainesosa(1); aineet.lisaa(maito12);
     *  Ainesosa maito23 = new Ainesosa(2); aineet.lisaa(maito23);
     *  Ainesosa maito51 = new Ainesosa(5); aineet.lisaa(maito51);
     *  
     *  List<Ainesosa> loytyneet;
     *  loytyneet = aineet.annaAinesosat(3);
     *  loytyneet.size() === 0; 
     *  loytyneet = aineet.annaAinesosat(1);
     *  loytyneet.size() === 2; 
     *  loytyneet.get(0) == maito11 === true;
     *  loytyneet.get(1) == maito12 === true;
     *  loytyneet = aineet.annaAinesosat(5);
     *  loytyneet.size() === 1; 
     *  loytyneet.get(0) == maito51 === true;
     * </pre> 
     */
    public List<Ainesosa> annaAinesosat(int tunnusnro) {
        List<Ainesosa> loydetyt = new ArrayList<Ainesosa>();
        for (Ainesosa ain : alkiot)
            if (ain.getId() == tunnusnro) loydetyt.add(ain);
        return loydetyt;
    }

    
    /**
     * Malliksi ainesosalista
     * @return malliainesosalista 
     */
    public List<Ainesosa> MalliAinesosat1() {
        List<Ainesosa> aineet1 = new ArrayList<Ainesosa>();
         Ainesosa aine10 = new Ainesosa();
         aine10.vastaaVehnajauho(1);
         aineet1.add(aine10);
         Ainesosa aine11 = new Ainesosa();
         aine11.vastaaMaito(1);
         aineet1.add(aine11);
         return aineet1;
    }
    
    
    /**
     * Malliksi toinen ainesosalista
     * @return toinen malliainesosalista
     */
    public List<Ainesosa> MalliAinesosat2() {
        List <Ainesosa> aineet2 = new ArrayList<Ainesosa>();
        Ainesosa aine20 = new Ainesosa();
        aine20.vastaaMaito(2);
        aineet2.add(aine20);
        Ainesosa aine21 = new Ainesosa();
        aine21.vastaaSuola(2);
        aineet2.add(aine21);
        return aineet2;
    }

    
    /**
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Ainesosat aineet = new Ainesosat();
        Ainesosa maito1 = new Ainesosa();
        maito1.vastaaMaito(2);
        Ainesosa maito2 = new Ainesosa();
        maito2.vastaaMaito(1);
        Ainesosa maito3 = new Ainesosa();
        maito3.vastaaMaito(2);
        Ainesosa maito4 = new Ainesosa();
        maito4.vastaaMaito(2);

        aineet.lisaa(maito1);
        aineet.lisaa(maito2);
        aineet.lisaa(maito3);
        aineet.lisaa(maito2);
        aineet.lisaa(maito4);

        System.out.println("============= Ainesosat testi =================");

        List<Ainesosa> Ainesosat2 = aineet.annaAinesosat(2);

        for (Ainesosa ain : Ainesosat2) {
            System.out.print(ain.getId() + " ");
            ain.tulosta(System.out);
        }
    
    }

}
