/**
 * 
 */
package reseptit;

import java.io.OutputStream;
import java.io.PrintStream;

/**
 * <pre>
* |------------------------------------------------------------------------|
* | Luokan nimi:    Ohje                              | Avustajat:         |
* |-------------------------------------------------------------------------
* | Vastuualueet:                                      |                   |
* |                                                    |                   |
* |(- ei tiedä reseptistä eikä käyttöliittymästä       |                   |
* | - tietää ohjeen kentät (nimi, kesto, määrä jne.)   |                   |
* | - osaa muuttaa 1| makaroonilaatikko|..|            |                   |
* | merkkijonoksi|                                     |                   |
* |-------------------------------------------------------------------------
 * </pre>
 * @author Omistaja
 * @version 13.3.2025
 *
 */
public class Ohje {

    private String nimi = "";
    private int id = 0;
    private String kesto = "";
    private int annoskoko;
    private String annostyyppi = "";
    private String teksti = "";
    
    private static int seuraavaId = 1;
    
    
    /**
     * Parametriton muodostaja
     */
    public Ohje() {
        valitseId();
    }
    
    
    /**
     * Ohjeen muodostaja jolle annetaan kaikki annettavat tiedot
     * @param nimi ohjeen nimi
     * @param annoskoko ohjeen tuottama annosmäärä
     * @param annostyyppi annoksen suuruutta kuvaava tyyppi
     * @param kesto kauanko ruuan teossa kestää
     * @param teksti ohjeen tekstiosa
     */
    public Ohje(String nimi, int annoskoko, String annostyyppi, String kesto, String teksti) {
        this.nimi = nimi;
        this.annoskoko = annoskoko;
        this.annostyyppi = annostyyppi;
        this.kesto = kesto;
        this.teksti = teksti;
        valitseId();
    }
    
    
    /**
     * Harjoittelua ja mallinnusta varten valmiiksi täytetty
     */
    public void Malliohje() {
        this.nimi = "Pinaattikeitto";
        this.kesto = "lyhyt";
        this.annoskoko = 2;
        this.annostyyppi = "annos";
        this.teksti = "Sulata voi kattilassa. Lisää jauhot hyvin sekoittaen. Anna kiehua hetken.\r\n"
                + "Lisää maito ohuena nauhana koko ajan sekoittaen. Kuumenna samalla kiehuvaksi.\r\n"
                + "Lisää pinaatti. Anna kiehua hiljalleen muutama minuutti välillä sekoittaen.\r\n"
                + "Mausta. Älä keitä suolan lisäämisen jälkeen.\r\n"
                + "Lisää lautaselle keiton pinnalle halkaistut, keitetyt munat.\r\n"
                + "Tarjoa pinaattikeitto ruisleivän kanssa.";
    }
    
    
    /**
     * Harjoittelua ja mallinnusta varten valmiiksi täytetty
     */
    public void Malliohje2() {
        this.nimi = "Makaroonilaatikko";
        this.kesto = "normaali";
        this.annoskoko = 4;
        this.annostyyppi = "annos";
        this.teksti = "Keitä makaronit pakkauksen ohjeen mukaan. Voit jättää makaronit myös keittämättä ja pääset näin helpommalla. Ruskista jauheliha ja sipuli pannulla paistamalla. Mausta kypsä jauheliha.\r\n"
                + "Kaada kerroksittain makaroneja ja jauhelihaa voideltuun vuokaan. Sekoita kananmunat ja maito (tai lihaliemi) ja kaada vuokaan. \r\n"
                + "Kypsennä 200-asteisen uunin alatasolla 1 tunti. Mikäli et esikeittänyt makaroneja, sekoita seosta kerran tai pari paiston alkupuolella/puolivälissä, jotta makaronit eivät jää painuneina vuoan pohjalle. ";
    }
    
    
    /**
     * Tulostaa ohjeen tiedot
     * @param os tietovirta mihin tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }
    
    
    /**
     * Tulostaa ohjeen tiedot
     * @param out tietovirta mihin tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println(String.format("%03d", id) + " " + nimi + ", " + kesto + ", " + annoskoko + ", " + annostyyppi + "\n" + teksti);
    }
    
    
    /**
     * Antaa ohjeelle tunnusnumeron
     * @return palauttaa ohjeen id:n
     * @example
     * <pre name="test">
     * Ohje makaronilaatikko3 = new Ohje();
     * Ohje makaronilaatikko4 = new Ohje();
     * makaronilaatikko3.getId() === 0;
     * makaronilaatikko3.lisaa();
     * makaronilaatikko4.lisaa();
     * int n1 = makaronilaatikko3.getId();
     * int n2 = makaronilaatikko4.getId();
     * n1 === n2-1;
     * </pre>
     */
    private int valitseId() {
        this.id = seuraavaId;
        seuraavaId++;
        return this.id;
    }
    
    
    /**
     * Poimii ohjeen numeron
     * @return palauttaa ohjeen numeron
     */
    public int getId() {
        return id;
    }
    
    
    /**
     * poimii ohjeen nimen
     * @return palauttaa ohjeen nimen
     */
    public String getNimi() {
        return nimi;
    }
    
    
    /**
     * @param ohje jonka annoskoko halutaan
     * @return annoskoko
     */
    public int getAnnoskoko(Ohje ohje) {
        return this.annoskoko;
    }
    
    
    /**
     * @param ohje jonka annostyyppi halutaan
     * @return annostyyppi
     */
    public String getAnnostyyppi(Ohje ohje) {
        return this.annostyyppi;
    }
    
    
    /**
     * @param ohje jonka kesto halutaan
     * @return kesto
     */
    public String getKesto(Ohje ohje) {
        return this.kesto;
    }
    
    
    /**
     * @param ohje jonka tekstiä halutaan
     * @return teksti
     */
    public String getTeksti(Ohje ohje) {
        return this.teksti;
    }    
    
    
    /**
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Ohje makaronilaatikko = new Ohje("Makaroonilaatikkko", 4, "pitkä", "annos", "Yhdistele aineet");
        Ohje makaronilaatikko2 = new Ohje("Makaroonilaatikko2", 4, "pitkä", "annos", "Yhdistele aineet");
        Ohje malli = new Ohje();
        malli.Malliohje();
        
        makaronilaatikko.tulosta(System.out);
        makaronilaatikko2.tulosta(System.out);
        malli.tulosta(System.out);
        
        makaronilaatikko.tulosta(System.out);
        makaronilaatikko2.tulosta(System.out);
        
    }

}
