/**
 * 
 */
package reseptit;

/**
 * <pre>
 *|------------------------------------------------------------------------|
 *| Luokan nimi:  Ohjeet                               | Avustajat:        |
 *|-------------------------------------------------------------------------
 *| Vastuualueet:                                      |                   |
 *|                                                    | - Ohje            |
 *| - Pitää listaa ohjeista, sekä osaa lisätä ja       |                   |
 *| poistaa ohjeita                                    |                   |
 *| - Lukee ja kirjoitaa ohjeista tiedostoon           |                   |
 *| - Osaa etsiä ja lajitella ohjeita                  |                   |
 *|-------------------------------------------------------------------------
 * </pre>
 * @author Omistaja
 * @version 18.3.2025
 *
 */
public class Ohjeet {
    
    private static final int MaxOhjeet = 5;
    private Ohje alkiot[] = new Ohje[MaxOhjeet];
    private int lkm = 0;
    private String tiedostonNimi = "";
    
    
    /**
     * Joku konstruktori
     */
    public Ohjeet() {
        
    }
    
    
    /**
     * Lisää uuden ohjeen tietorakenteeseen.  Ottaa ohjeen omistukseensa.
     * @param ohje lisätäävän ohjeeseen viite.  Huom tietorakenne muuttuu omistajaksi
     * @throws SailoException jos tietorakenne on jo täynnä
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * Ohjeet ohjeet = new Ohjeet();
     * Ohje makaronilaatikko1 = new Ohje(), makaronilaatikko2 = new Ohje();
     * ohjeet.getLkm() === 0;
     * ohjeet.lisaa(makaronilaatikko1); ohjeet.getLkm() === 1;
     * ohjeet.lisaa(makaronilaatikko2); ohjeet.getLkm() === 2;
     * ohjeet.lisaa(makaronilaatikko1); ohjeet.getLkm() === 3;
     * ohjeet.anna(0) === makaronilaatikko1;
     * ohjeet.anna(1) === makaronilaatikko2;
     * ohjeet.anna(2) === makaronilaatikko1;
     * ohjeet.anna(1) == makaronilaatikko1 === false;
     * ohjeet.anna(1) == makaronilaatikko2 === true;
     * ohjeet.anna(3) === makaronilaatikko1; #THROWS IndexOutOfBoundsException 
     * ohjeet.lisaa(makaronilaatikko1); ohjeet.getLkm() === 4;
     * ohjeet.lisaa(makaronilaatikko1); ohjeet.getLkm() === 5;
     * ohjeet.lisaa(makaronilaatikko1);  #THROWS SailoException
     * </pre>
     */
    public void lisaa(Ohje ohje) throws SailoException {
        if (lkm >= alkiot.length) throw new SailoException("Liikaa alkioita");
        alkiot[lkm] = ohje;
        lkm++;
    }
    
    
    /**
     * Palauttaa viitteen i:teen ohjeeseen.
     * @param i monennenko ohjeen viite halutaan
     * @return viite ohjeeseen, jonka indeksi on i
     * @throws IndexOutOfBoundsException jos i ei ole sallitulla alueella  
     */
    public Ohje anna(int i) throws IndexOutOfBoundsException {
        if (i < 0 || lkm <= i)
            throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        return alkiot[i];
    }


    /**
     * Lukee Ohjeet tiedostosta.
     * @param hakemisto tiedoston hakemisto
     * @throws SailoException jos lukeminen epäonnistuu
     */
    public void lueTiedostosta(String hakemisto) throws SailoException {
        tiedostonNimi = hakemisto + "/ohjeet.dat";
        throw new SailoException("Ei osata vielä lukea tiedostoa " + tiedostonNimi);
    }


    /**
     * Tallentaa ohjeet tiedostoon.
     * @throws SailoException jos talletus epäonnistuu
     */
    public void talleta() throws SailoException {
        throw new SailoException("Ei osata vielä tallettaa tiedostoa " + tiedostonNimi);
    }


    /**
     * Palauttaa Reseptien lukumäärän
     * @return ohjeiden lukumäärä
     */
    public int getLkm() {
        return lkm;
    }

    
    /**
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Ohjeet ohjeet = new Ohjeet();

        Ohje makaronilaatikko = new Ohje("makaronilaatikko", 9, "annos", "pitkä", "sekoita aineet"), makaronilaatikko2 = new Ohje("makaronilaatikko", 9, "annos", "pitkä", "sekoita aineet");
       


        System.out.println("============= Ohjeet testi =================");
        System.out.println(makaronilaatikko);
        System.out.println(makaronilaatikko2);
        for (int i = 0; i < ohjeet.getLkm(); i++) {
            Ohje ohje = ohjeet.anna(i);
            System.out.println("Ohjeetn nro: " + i);
            ohje.tulosta(System.out);
        }

    }

}
