/**
 * 
 */
package reseptit;

import java.util.List;

/**
 * <pre>
 *|------------------------------------------------------------------------|
 *| Luokan nimi:  Reseptit                             | Avustajat:        |
 *|-------------------------------------------------------------------------
 *| Vastuualueet:                                      |                   |
 *|                                                    | - Ohje            |
 *| - Huolehtii Ainesosat ja Ohjeet välisestä          | - Ohjeet          |
 *| yhteistyöstä ja välittää näiden tietoja            | - Ainesosa        |
 *| pyydettäessä                                       | - Ainesosat       |
 *|-------------------------------------------------------------------------
 * </pre>
 * @author Omistaja
 * @version 18.3.2025
 *
 */
public class Reseptit {

    private final Ohjeet ohjeet = new Ohjeet();
    private final Ainesosat ainesosat = new Ainesosat();
   
    

    /**
     * Listään uusi ainesosa resepteihin
     * @param ain lisättävä ainesosa
     */
    public void lisaa(Ainesosa ain) {
        ainesosat.lisaa(ain);
    }

    
    /**
     * Palautaa ohjeiden määrän
     * @return ohjeiden määrä
     */
    public int getLkm() {
        return ohjeet.getLkm();
    }
    
    
    /**
     * Getteri ohjeen tekstille
     * @param ohje ohje, jonka teksti halutaan palauttaa
     * @return ohjeen tekstiosa
     */
    public String getNimi(Ohje ohje) {
        return ohje.getNimi();
    }
    
    
    /**
     * Getteri ohjeen tekstille
     * @param ohje ohje, jonka teksti halutaan palauttaa
     * @return ohjeen tekstiosa
     */
    public String getKesto(Ohje ohje) {
        return ohje.getKesto(ohje);
    }
    
    
    /**
     * Getteri ohjeen tekstille
     * @param ohje ohje, jonka teksti halutaan palauttaa
     * @return ohjeen tekstiosa
     */
    public int getAnnoskoko(Ohje ohje) {
        return ohje.getAnnoskoko(ohje);
    }
    
    
    /**
     * Getteri ohjeen tekstille
     * @param ohje ohje, jonka teksti halutaan palauttaa
     * @return ohjeen tekstiosa
     */
    public String getAnnostyyppi(Ohje ohje) {
        return ohje.getAnnostyyppi(ohje);
    }
    
    
    /**
     * Getteri ohjeen tekstille
     * @param ohje ohje, jonka teksti halutaan palauttaa
     * @return ohjeen tekstiosa
     */
    public String getTeksti(Ohje ohje) {
        return ohje.getTeksti(ohje);
    }
    
    
    /**
     * Palautaa ainesosien määrän
     * @return ainesosien määrä
     */
    public int getALkm() {
        return ainesosat.getALkm();
    }
    
    
    /**
     * @param ain kysyttävä ainesosa
     * @return palauttaa ainesosan nimen
     */
    public String getAnimi(Ainesosa ain) {
        return ain.getNimi(ain);
    }
    
    
    /**
     * @param ain kysyttävä ainesosa
     * @return määrä
     */
    public double getAmaara(Ainesosa ain) {
        return ain.getMaara(ain);
    }
    
    
    
    /**
     * @param ain kysyttävä ainesosa
     * @return yksikkö
     */
    public String getAyksikko(Ainesosa ain) {
        return ain.getYksikko(ain);
    }


    /**
     * Poistaa ohjeista ne joilla on nro. Kesken.
     * @param nro viitenumero, jonka mukaan poistetaan
     * @return montako ohjetta poistettiin
     */
    public int poista(@SuppressWarnings("unused") int nro) {
        return 0;
    }


    /**
     * Lisää resepteihin uuden ohjeen
     * @param ohje lisättävä ohje
     * @throws SailoException jos lisäystä ei voida tehdä
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * Reseptit reseptit = new Reseptit();
     * Ohje makaronilaatikko1 = new Ohje(), makaronilaatikko2 = new Ohje();
     * makaronilaatikko1.lisaa(); makaronilaatikko2.lisaa();
     * reseptit.getJasenia() === 0;
     * reseptit.lisaa(makaronilaatikko1); reseptit.getOhjeet() === 1;
     * reseptit.lisaa(makaronilaatikko2); reseptit.getOhjeet() === 2;
     * reseptit.lisaa(makaronilaatikko1); reseptit.getOhjeet() === 3;
     * reseptit.getOhjeet() === 3;
     * reseptit.annaOhje(0) === makaronilaatikko1;
     * reseptit.annaOhje(1) === makaronilaatikko2;
     * reseptit.annaOhje(2) === makaronilaatikko1;
     * reseptit.annaOhje(3) === makaronilaatikko1; #THROWS IndexOutOfBoundsException 
     * reseptit.lisaa(makaronilaatikko1); reseptit.getOhjeet() === 4;
     * reseptit.lisaa(makaronilaatikko1); reseptit.getOhjeet() === 5;
     * reseptit.lisaa(makaronilaatikko1);            #THROWS SailoException
     * </pre>
     */
    public void lisaa(Ohje ohje) throws SailoException {
        ohjeet.lisaa(ohje);
    }


    /**
     * Palauttaa i:n ohjeen
     * @param i monesko ohje palautetaan
     * @return viite i:teen ohjeeseen
     * @throws IndexOutOfBoundsException jos i väärin
     */
    public Ohje annaOhje(int i) throws IndexOutOfBoundsException {
        return ohjeet.anna(i);
    }


    /**
     * Haetaan kaikki ohjeen ainesosat
     * @param ohje ohje jolle ainesosat haetaan
     * @return tietorakenne jossa viiteet löydetteyihin ainesosiin
     * @example
     * <pre name="test">
     * #import java.util.*;
     * 
     *  Reseptit reseptit = new Reseptit();
     *  Ohje makaroonilaatikko1 = new Ohje(), makaroonilaatikko2 = new Ohje(), makaroonilaatikko3 = new Ohje();
     *  makaroonilaatikko1.valitseId(); makaroonilaatikko2.valitseId(); makaroonilaatiikko3.valitseId();
     *  int id1 = makaroonilaatikko1.getId();
     *  int id2 = makaroonilaatikko2.getId();
     *  Ainesosa maito11 = new Ainesosa(id1); reseptit.lisaa(maito11);
     *  Ainesosa maito12 = new Ainesosa(id1); reseptit.lisaa(maito12);
     *  Ainesosa maito21 = new Ainesosa(id2); reseptit.lisaa(maito21);
     *  Ainesosa maito22 = new Ainesosa(id2); reseptit.lisaa(maito22);
     *  Ainesosa maito23 = new Ainesosa(id2); reseptit.lisaa(maito23);
     *  
     *  List<Ainesosa> loytyneet;
     *  loytyneet = reseptit.annaAinesosat(makaroonilaatikko3);
     *  loytyneet.size() === 0; 
     *  loytyneet = reseptit.annaAinesosat(makaroonilaatikko1);
     *  loytyneet.size() === 2; 
     *  loytyneet.get(0) == maito11 === true;
     *  loytyneet.get(1) == maito12 === true;
     *  loytyneet = reseptit.annaHarrastukset(makaroonilaatikko2);
     *  loytyneet.size() === 3; 
     *  loytyneet.get(0) == maito21 === true;
     * </pre> 
     */
    public List<Ainesosa> annaAinesosat(Ohje ohje) {
        String ohjeennimi = ohje.getNimi();
        if (ohjeennimi.equals("Pinaattikeitto")) {
            return ainesosat.MalliAinesosat1();
        }
        return ainesosat.MalliAinesosat2(); 
        
    }
    
    
    /**
     * @param ohje ohje johon ainesosat liittyy
     * @return palauttaaa ainesosat
     */
    public List<Ainesosa> TestiAinesosat1(Ohje ohje){
        return ainesosat.MalliAinesosat1();
    }
    
    
    /**
     * @param ohje ohje johon ainesosat liittyy
     * @return palauttaaa ainesosat
     */
    public List<Ainesosa> TestiAinesosat2(Ohje ohje){
        return ainesosat.MalliAinesosat2();
    }
    
    
    /**
     * Lukee ohjeen tiedot tiedostosta
     * @param nimi jota käyteään lukemisessa
     * @throws SailoException jos lukeminen epäonnistuu
     */
    public void lueTiedostosta(String nimi) throws SailoException {
        ohjeet.lueTiedostosta(nimi);
        ainesosat.lueTiedostosta(nimi);
    }


    /**
     * Tallettaa ohjeen tiedot tiedostoon
     * @throws SailoException jos tallettamisessa ongelmia
     */
    public void talleta() throws SailoException {
        ohjeet.talleta();
        ainesosat.talleta();
        // TODO: yritä tallettaa toinen vaikka toinen epäonnistuisi
    }

    
    /**
     * Testiohjelma Reseptille
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Reseptit reseptit = new Reseptit();

        Ohje makaronilaatikko1 = new Ohje(), makaronilaatikko2 = new Ohje();
        
        //makaronilaatikko1.vastaaAkuAnkka();
        
        //makaronilaatikko2.vastaaAkuAnkka();



        System.out.println("============= Reseptit testi =================");
        System.out.println(makaronilaatikko1);
        System.out.println(makaronilaatikko2);
        for (int i = 0; i < reseptit.ohjeet.getLkm(); i++) {
            Ohje ohje = reseptit.annaOhje(i);
            System.out.println("Ohje kohdassa: " + i);
            ohje.tulosta(System.out);
            List<Ainesosa> loytyneet = reseptit.annaAinesosat(ohje);
            for (Ainesosa ainesosa : loytyneet)
                ainesosa.tulosta(System.out);

        }

    
    }

}
